@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP #-}
 -- |
 -- Module    : Statistics.Resampling.Bootstrap
 -- Copyright : (c) 2009, 2011 Bryan O'Sullivan
@@ -31,9 +30,7 @@ import Statistics.Function (gsort)
 
 import qualified Statistics.Resampling as R
 
-#if !defined(__GHCJS__)
-import Control.Monad.Par (parMap, runPar)
-#endif
+import Control.Parallel.Strategies (parMap, rdeepseq)
 
 data T = {-# UNPACK #-} !Double :< {-# UNPACK #-} !Double
 infixl 2 :<
@@ -51,15 +48,7 @@ bootstrapBCA
   --   this.
   -> [Estimate ConfInt Double]
 bootstrapBCA confidenceLevel sample resampledData
-#if defined(__GHCJS__)
-  -- monad-par causes seems to cause "thread blocked indefinitely on MVar"
-  -- on GHCJS still
-  --
-  -- I (phadej) would change the interface to return IO, and use mapConcurrently from async
-  = map e resampledData
-#else
-  = runPar $ parMap e resampledData
-#endif
+  = parMap rdeepseq e resampledData
   where
     e (est, Bootstrap pt resample)
       | U.length sample == 1 || isInfinite bias =
